@@ -12,7 +12,11 @@
 
 #include "auth.h"
 #include "bt-info.h"
+#include "calls-manager.h"
 #include "lockscreen.h"
+#include "notifications/notify-manager.h"
+#include "notifications/notification-frame.h"
+#include "util.h"
 
 #include <locale.h>
 #include <string.h>
@@ -20,8 +24,9 @@
 #include <math.h>
 #include <time.h>
 
-#define HANDY_USE_UNSTABLE_API
 #include <handy.h>
+#include <cui-call-display.h>
+
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-wall-clock.h>
@@ -35,8 +40,25 @@
  *
  * The lock screen featuring the clock
  * and unlock keypad.
+ *
+ * # CSS nodes
+ *
+ * #PhoshLockscreen has a CSS name with the name phosh-lockscreen.
  */
 
+
+typedef enum {
+  POS_OVERVIEW = 0,
+  POS_UNLOCK   = 1,
+} PhoshLocksreenPos;
+
+enum {
+  PROP_0,
+  PROP_CALLS_MANAGER,
+  PROP_LAST_PROP
+};
+static GParamSpec *props[PROP_LAST_PROP];
+
 enum {
   LOCKSCREEN_UNLOCK,
   WAKEUP_OUTPUT,
@@ -44,7 +66,6 @@ enum {
 };
 static guint signals[N_SIGNALS] = { 0 };
 
-
 typedef struct _PhoshLockscreen
 {
   PhoshLayerSurface parent;
@@ -52,33 +73,84 @@ typedef struct _PhoshLockscreen
 
 
 typedef struct {
-  GtkWidget *carousel;
+  GtkWidget         *carousel;
 
   /* info page */
-  GtkWidget *box_info;
-  GtkWidget *lbl_clock;
-  GtkWidget *lbl_date;
+  GtkWidget         *box_info;
+  GtkWidget         *lbl_clock;
+  GtkWidget         *lbl_date;
+  GtkWidget         *list_notifications;
+  GtkWidget         *sw_notifications;
+  GSettings         *settings;
 
   /* unlock page */
-  GtkWidget *box_unlock;
-  GtkWidget *keypad;
-  GtkWidget  *entry_pin;
-  GtkGesture *long_press_del_gesture;
-  GtkWidget *lbl_unlock_status;
-  GtkWidget *btn_submit;
-  GtkWidget *btn_emergency;
-  guint      idle_timer;
-  gint64     last_input;
-  PhoshAuth *auth;
+  GtkWidget         *box_unlock;
+  GtkWidget         *keypad;
+  GtkWidget         *entry_pin;
+  GtkGesture        *long_press_del_gesture;
+  GtkWidget         *lbl_unlock_status;
+  GtkWidget         *btn_submit;
+  GtkWidget         *btn_emergency;
+  guint              idle_timer;
+  gint64             last_input;
+  PhoshAuth         *auth;
 
-  GnomeWallClock *wall_clock;
+  /* Call page */
+  HdyDeck           *deck;
+  GtkBox            *box_call_display;
+  CuiCallDisplay    *call_display;
+
+  GnomeWallClock    *wall_clock;
+  PhoshCallsManager *calls_manager;
+  char              *active; /* opaque handle to the active call */
 } PhoshLockscreenPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (PhoshLockscreen, phosh_lockscreen, PHOSH_TYPE_LAYER_SURFACE)
 
 
 static void
-clear_input (PhoshLockscreen *self, gboolean clear_all) {
+phosh_lockscreen_set_property (GObject      *object,
+                               guint         property_id,
+                               const GValue *value,
+                               GParamSpec   *pspec)
+{
+  PhoshLockscreen *self = PHOSH_LOCKSCREEN (object);
+  PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
+
+  switch (property_id) {
+  case PROP_CALLS_MANAGER:
+    priv->calls_manager = g_value_dup_object (value);
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    break;
+  }
+}
+
+
+static void
+phosh_lockscreen_get_property (GObject    *object,
+                               guint       property_id,
+                               GValue     *value,
+                               GParamSpec *pspec)
+{
+  PhoshLockscreen *self = PHOSH_LOCKSCREEN (object);
+  PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
+
+  switch (property_id) {
+  case PROP_CALLS_MANAGER:
+    g_value_set_object (value, priv->calls_manager);
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    break;
+  }
+}
+
+
+static void
+clear_input (PhoshLockscreen *self, gboolean clear_all)
+{
   PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
 
   if (clear_all) {
@@ -94,6 +166,7 @@ static void
 show_info_page (PhoshLockscreen *self)
 {
   PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
+
   if (hdy_carousel_get_position (HDY_CAROUSEL (priv->carousel)) <= 0)
     return;
 
@@ -121,6 +194,7 @@ static void
 show_unlock_page (PhoshLockscreen *self)
 {
   PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
+
   if (hdy_carousel_get_position (HDY_CAROUSEL (priv->carousel)) > 0)
     return;
 
@@ -133,7 +207,8 @@ show_unlock_page (PhoshLockscreen *self)
 
 
 static gboolean
-finish_shake_label (PhoshLockscreen *self) {
+finish_shake_label (PhoshLockscreen *self)
+{
   clear_input (self, TRUE);
   gtk_widget_set_sensitive (GTK_WIDGET (self), TRUE);
   return FALSE;
@@ -141,9 +216,10 @@ finish_shake_label (PhoshLockscreen *self) {
 
 
 static gboolean
-shake_label (GtkWidget *widget,
-                         GdkFrameClock *frame_clock,
-                         gpointer data) {
+shake_label (GtkWidget     *widget,
+             GdkFrameClock *frame_clock,
+             gpointer       data)
+{
   PhoshLockscreen *self = PHOSH_LOCKSCREEN (widget);
   PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
   gint64 start_time = g_variant_get_int64 (data);
@@ -184,7 +260,7 @@ auth_async_cb (PhoshAuth *auth, GAsyncResult *result, PhoshLockscreen *self)
 
   g_object_ref (self);
   if (authenticated) {
-    g_signal_emit(self, signals[LOCKSCREEN_UNLOCK], 0);
+    g_signal_emit (self, signals[LOCKSCREEN_UNLOCK], 0);
     g_clear_object (&priv->auth);
   } else {
     GdkFrameClock *clock;
@@ -206,7 +282,7 @@ auth_async_cb (PhoshAuth *auth, GAsyncResult *result, PhoshLockscreen *self)
 
 static void
 delete_button_clicked_cb (PhoshLockscreen *self,
-                          GtkWidget      *widget)
+                          GtkWidget       *widget)
 {
   g_return_if_fail (PHOSH_IS_LOCKSCREEN (self));
 
@@ -381,14 +457,14 @@ local_date (void)
 
 static void
 wall_clock_notify_cb (PhoshLockscreen *self,
-                      GParamSpec *pspec,
-                      GnomeWallClock *wall_clock)
+                      GParamSpec      *pspec,
+                      GnomeWallClock  *wall_clock)
 {
   PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
   const char *time;
   g_autofree char *date = NULL;
 
-  time = gnome_wall_clock_get_clock(wall_clock);
+  time = gnome_wall_clock_get_clock (wall_clock);
   gtk_label_set_text (GTK_LABEL (priv->lbl_clock), time);
 
   date = local_date ();
@@ -406,12 +482,12 @@ carousel_position_notified_cb (PhoshLockscreen *self,
 
   position = hdy_carousel_get_position (HDY_CAROUSEL (priv->carousel));
 
-  if (position <= 0) {
+  if (position <= POS_OVERVIEW) {
     clear_input (self, TRUE);
     return;
   }
 
-  if (position >= 1) {
+  if (position >= POS_UNLOCK) {
     if (!priv->idle_timer) {
       priv->last_input = g_get_monotonic_time ();
       priv->idle_timer = g_timeout_add_seconds (LOCKSCREEN_IDLE_SECONDS,
@@ -429,22 +505,110 @@ carousel_position_notified_cb (PhoshLockscreen *self,
 }
 
 
+static void
+on_calls_call_inbound (PhoshLockscreen *self, const gchar *path)
+{
+  PhoshLockscreenPrivate *priv;
+  PhoshCall *call;
+
+  g_return_if_fail (PHOSH_IS_LOCKSCREEN (self));
+  priv = phosh_lockscreen_get_instance_private (self);
+  g_return_if_fail (PHOSH_IS_CALLS_MANAGER (priv->calls_manager));
+
+  g_debug ("New inbound call %s", path);
+  g_signal_emit (self, signals[WAKEUP_OUTPUT], 0);
+
+  g_free (priv->active);
+  priv->active = g_strdup (path);
+
+  call = phosh_calls_manager_get_call (priv->calls_manager, path);
+  g_return_if_fail (PHOSH_IS_CALL (call));
+
+  hdy_deck_set_visible_child (priv->deck, GTK_WIDGET (priv->box_call_display));
+
+  cui_call_display_set_call (priv->call_display, CUI_CALL (call));
+}
+
+
+static void
+on_calls_call_removed (PhoshLockscreen *self, const gchar *path)
+{
+  PhoshLockscreenPrivate *priv;
+
+  g_return_if_fail (path != NULL);
+  g_return_if_fail (PHOSH_IS_LOCKSCREEN (self));
+  priv = phosh_lockscreen_get_instance_private (self);
+
+  g_debug ("Call %s removed, active: %s", path, priv->active);
+
+  if (g_strcmp0 (path, priv->active))
+    return;
+
+  g_clear_pointer (&priv->active, g_free);
+  hdy_deck_navigate (priv->deck, HDY_NAVIGATION_DIRECTION_BACK);
+}
+
+
+static GtkWidget *
+create_notification_row (gpointer item, gpointer data)
+{
+  GtkWidget *row = NULL;
+  GtkWidget *frame = NULL;
+
+  row = g_object_new (GTK_TYPE_LIST_BOX_ROW,
+                      "activatable", FALSE,
+                      "visible", TRUE,
+                      NULL);
+
+  frame = phosh_notification_frame_new (FALSE);
+  phosh_notification_frame_bind_model (PHOSH_NOTIFICATION_FRAME (frame), item);
+
+  gtk_widget_show (frame);
+  gtk_container_add (GTK_CONTAINER (row), frame);
+
+  return row;
+}
+
+
+static void
+on_notifcation_items_changed (PhoshLockscreen *self,
+                              guint            position,
+                              guint            removed,
+                              guint            added,
+                              GListModel      *list)
+{
+  PhoshLockscreenPrivate *priv;
+  gboolean is_empty;
+
+  g_return_if_fail (G_IS_LIST_MODEL (list));
+  g_return_if_fail (PHOSH_IS_LOCKSCREEN (self));
+  priv = phosh_lockscreen_get_instance_private (self);
+
+  is_empty = !g_list_model_get_n_items (list);
+  g_debug("Notification list empty: %d", is_empty);
+
+  /* Don't unhide when we don't want notification on the lock screen */
+  if (!is_empty && !g_settings_get_boolean (priv->settings, "show-in-lock-screen"))
+    return;
+
+  gtk_widget_set_visible (GTK_WIDGET (priv->sw_notifications), !is_empty);
+}
+
+
 static void
 phosh_lockscreen_constructed (GObject *object)
 {
   PhoshLockscreen *self = PHOSH_LOCKSCREEN (object);
   PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
+  const char *active;
+  PhoshNotifyManager *manager;
 
   G_OBJECT_CLASS (phosh_lockscreen_parent_class)->constructed (object);
 
   /* window properties */
   gtk_window_set_title (GTK_WINDOW (self), "phosh lockscreen");
   gtk_window_set_decorated (GTK_WINDOW (self), FALSE);
-  gtk_widget_realize(GTK_WIDGET (self));
-
-  gtk_style_context_add_class (
-      gtk_widget_get_style_context (GTK_WIDGET (self)),
-      "phosh-lockscreen");
+  gtk_widget_realize (GTK_WIDGET (self));
 
   gtk_widget_add_events (GTK_WIDGET (self), GDK_KEY_PRESS_MASK);
   g_signal_connect (G_OBJECT (self),
@@ -460,6 +624,50 @@ phosh_lockscreen_constructed (GObject *object)
                             G_CALLBACK (wall_clock_notify_cb),
                             self);
   wall_clock_notify_cb (self, NULL, priv->wall_clock);
+
+  g_signal_connect_object (priv->calls_manager,
+                           "call-inbound",
+                           G_CALLBACK (on_calls_call_inbound),
+                           self,
+                           G_CONNECT_SWAPPED);
+  g_signal_connect_object (priv->calls_manager,
+                           "call-removed",
+                           G_CALLBACK (on_calls_call_removed),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  /* If a call is ongoing show it when locking until we show a notification */
+  active = phosh_calls_manager_get_active_call_handle (priv->calls_manager);
+  if (active)
+    on_calls_call_inbound (self, active);
+
+  manager = phosh_notify_manager_get_default ();
+  /* TODO: deduplicate after !862 */
+  priv->settings = g_settings_new("org.gnome.desktop.notifications");
+  g_settings_bind (priv->settings, "show-in-lock-screen",
+                   priv->list_notifications, "visible",
+                   G_SETTINGS_BIND_GET);
+  gtk_list_box_bind_model (GTK_LIST_BOX (priv->list_notifications),
+                           G_LIST_MODEL (phosh_notify_manager_get_list (manager)),
+                           create_notification_row,
+                           NULL,
+                           NULL);
+  g_signal_connect_object (phosh_notify_manager_get_list (manager),
+                           "items-changed",
+                           G_CALLBACK (on_notifcation_items_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
+  on_notifcation_items_changed (self, -1, -1, -1,
+                                G_LIST_MODEL (phosh_notify_manager_get_list (manager)));
+}
+
+static void
+deck_back_clicked_cb (GtkWidget       *sender,
+                      PhoshLockscreen *self)
+{
+  PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
+
+  hdy_deck_navigate (priv->deck, HDY_NAVIGATION_DIRECTION_BACK);
 }
 
 
@@ -469,11 +677,11 @@ phosh_lockscreen_dispose (GObject *object)
   PhoshLockscreen *self = PHOSH_LOCKSCREEN (object);
   PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
 
+  g_clear_object (&priv->settings);
   g_clear_object (&priv->wall_clock);
-  if (priv->idle_timer) {
-    g_source_remove (priv->idle_timer);
-    priv->idle_timer = 0;
-  }
+  g_clear_handle_id (&priv->idle_timer, g_source_remove);
+  g_clear_object (&priv->calls_manager);
+  g_clear_pointer (&priv->active, g_free);
 
   G_OBJECT_CLASS (phosh_lockscreen_parent_class)->dispose (object);
 }
@@ -488,9 +696,21 @@ phosh_lockscreen_class_init (PhoshLockscreenClass *klass)
   object_class->constructed = phosh_lockscreen_constructed;
   object_class->dispose = phosh_lockscreen_dispose;
 
+  object_class->set_property = phosh_lockscreen_set_property;
+  object_class->get_property = phosh_lockscreen_get_property;
+
+  props[PROP_CALLS_MANAGER] =
+    g_param_spec_object ("calls-manager",
+                         "",
+                         "",
+                         PHOSH_TYPE_CALLS_MANAGER,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
+
+  g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
+
   signals[LOCKSCREEN_UNLOCK] = g_signal_new ("lockscreen-unlock",
-      G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      NULL, G_TYPE_NONE, 0);
+                                             G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+                                             NULL, G_TYPE_NONE, 0);
   /**
    * PhoshLockscreen::wakeup-output
    * @self: The #PhoshLockscreen emitting this signal
@@ -499,8 +719,8 @@ phosh_lockscreen_class_init (PhoshLockscreenClass *klass)
    * up.
    */
   signals[WAKEUP_OUTPUT] = g_signal_new ("wakeup-output",
-      G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      NULL, G_TYPE_NONE, 0);
+                                         G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+                                         NULL, G_TYPE_NONE, 0);
 
   gtk_widget_class_set_css_name (widget_class, "phosh-lockscreen");
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -508,7 +728,7 @@ phosh_lockscreen_class_init (PhoshLockscreenClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, carousel);
   gtk_widget_class_bind_template_callback_full (widget_class,
                                                 "carousel_position_notified_cb",
-                                                G_CALLBACK(carousel_position_notified_cb));
+                                                G_CALLBACK (carousel_position_notified_cb));
 
   /* unlock page */
   gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, box_unlock);
@@ -528,7 +748,15 @@ phosh_lockscreen_class_init (PhoshLockscreenClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, box_info);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, lbl_clock);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, lbl_date);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, list_notifications);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, sw_notifications);
   gtk_widget_class_bind_template_callback (widget_class, show_unlock_page);
+
+  /* Call UI */
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, deck);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, box_call_display);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshLockscreen, call_display);
+  gtk_widget_class_bind_template_callback (widget_class, deck_back_clicked_cb);
 }
 
 
@@ -541,7 +769,8 @@ phosh_lockscreen_init (PhoshLockscreen *self)
 
 GtkWidget *
 phosh_lockscreen_new (gpointer layer_shell,
-                      gpointer wl_output)
+                      gpointer wl_output,
+                      PhoshCallsManager *calls_manager)
 {
   return g_object_new (PHOSH_TYPE_LOCKSCREEN,
                        "layer-shell", layer_shell,
@@ -554,6 +783,7 @@ phosh_lockscreen_new (gpointer layer_shell,
                        "kbd-interactivity", TRUE,
                        "exclusive-zone", -1,
                        "namespace", "phosh lockscreen",
+                       "calls-manager", calls_manager,
                        NULL);
 }
 
@@ -566,11 +796,36 @@ phosh_lockscreen_new (gpointer layer_shell,
 PhoshLockscreenPage
 phosh_lockscreen_get_page (PhoshLockscreen *self)
 {
-  PhoshLockscreenPrivate *priv = phosh_lockscreen_get_instance_private (self);
-  gdouble position = hdy_carousel_get_position (HDY_CAROUSEL (priv->carousel));
+  PhoshLockscreenPrivate *priv;
+  gdouble position;
+
+  g_return_val_if_fail (PHOSH_IS_LOCKSCREEN (self), PHOSH_LOCKSCREEN_PAGE_DEFAULT);
+  priv = phosh_lockscreen_get_instance_private (self);
+  position = hdy_carousel_get_position (HDY_CAROUSEL (priv->carousel));
 
   if (position <= 0)
     return PHOSH_LOCKSCREEN_PAGE_DEFAULT;
   else
     return PHOSH_LOCKSCREEN_PAGE_UNLOCK;
 }
+
+/*
+ * phosh_lockscreen_set_page
+ * @self: The #PhoshLockscreen
+ * PhoshLockscreenPage: the page to scroll to
+ *
+ * Scrolls to a specific page in the carousel. The state of the deck isn't changed.
+ */
+void
+phosh_lockscreen_set_page (PhoshLockscreen *self, PhoshLockscreenPage page)
+{
+  GtkWidget *scroll_to;
+  PhoshLockscreenPrivate *priv;
+
+  g_return_if_fail (PHOSH_IS_LOCKSCREEN (self));
+  priv = phosh_lockscreen_get_instance_private (self);
+
+  scroll_to = (page == PHOSH_LOCKSCREEN_PAGE_UNLOCK) ? priv->box_unlock : priv->box_info;
+
+  hdy_carousel_scroll_to (HDY_CAROUSEL (priv->carousel), scroll_to);
+}
