@@ -89,8 +89,8 @@ output_handle_scale (void             *data,
 {
   PhoshMonitor *self = PHOSH_MONITOR (data);
 
+  /* nothing to do */
   self->wl_output_done = FALSE;
-  self->scale = scale;
 }
 
 
@@ -168,7 +168,6 @@ xdg_output_v1_handle_logical_size (void *data,
   g_debug ("Monitor %p: Logical size: %dx%d", self, width, height);
   self->logical.width = width;
   self->logical.height = height;
-
 }
 
 
@@ -259,7 +258,7 @@ static void
 wlr_output_power_handle_failed(void *data,
                                struct zwlr_output_power_v1 *output_power)
 {
-  PhoshMonitor *self = data;
+  PhoshMonitor *self = PHOSH_MONITOR (data);
 
   g_return_if_fail (PHOSH_IS_MONITOR (self));
   g_warning("Failed to set output power mode for %s\n", self->name);
@@ -405,7 +404,6 @@ phosh_monitor_class_init (PhoshMonitorClass *klass)
 static void
 phosh_monitor_init (PhoshMonitor *self)
 {
-  self->scale = 1.0;
   self->modes = g_array_new (FALSE, FALSE, sizeof(PhoshMonitorMode));
   self->power_mode = PHOSH_MONITOR_POWER_SAVE_MODE_OFF;
 }
@@ -609,3 +607,32 @@ phosh_monitor_get_wl_output (PhoshMonitor *self)
 
   return self->wl_output;
 }
+
+/**
+ * phosh_monitor_get_fractional_scale:
+ * @self: The monitor
+ *
+ * Get the fractinoal scale determined from the output width and the
+ * current logical width.
+ * Returns: the fractional scale
+*/
+float
+phosh_monitor_get_fractional_scale (PhoshMonitor *self)
+{
+  float width;
+
+  g_return_val_if_fail (PHOSH_IS_MONITOR (self), 1.0);
+  g_return_val_if_fail (phosh_monitor_is_configured (self), 1.0);
+
+  switch (self->transform) {
+  case PHOSH_MONITOR_TRANSFORM_NORMAL:
+  case PHOSH_MONITOR_TRANSFORM_180:
+  case PHOSH_MONITOR_TRANSFORM_FLIPPED:
+  case PHOSH_MONITOR_TRANSFORM_FLIPPED_180:
+    width = self->logical.width;
+    break;
+  default:
+    width = self->logical.height;
+  }
+  return self->width / width;
+}
