@@ -37,7 +37,7 @@ static guint signals[N_SIGNALS] = { 0 };
 
 struct _PhoshNetworkAuthPrompt
 {
-  PhoshLayerSurface parent;
+  PhoshSystemModalDialog parent;
 
   GtkWidget      *cancel_button;
   GtkWidget      *connect_button;
@@ -62,7 +62,7 @@ struct _PhoshNetworkAuthPrompt
   gboolean visible; /* is input visible */
 };
 
-G_DEFINE_TYPE(PhoshNetworkAuthPrompt, phosh_network_auth_prompt, PHOSH_TYPE_LAYER_SURFACE);
+G_DEFINE_TYPE(PhoshNetworkAuthPrompt, phosh_network_auth_prompt, PHOSH_TYPE_SYSTEM_MODAL_DIALOG);
 
 
 static void
@@ -216,11 +216,12 @@ network_prompt_setup_dialog (PhoshNetworkAuthPrompt *self)
 
     gtk_entry_buffer_set_text (GTK_ENTRY_BUFFER (self->password_buffer), password, -1);
   }
+  gtk_widget_grab_focus (self->wpa_password_entry);
 }
 
 
 static void
-network_prompt_cancel_clicked_cb (PhoshNetworkAuthPrompt *self)
+on_dialog_canceled (PhoshNetworkAuthPrompt *self)
 {
   g_return_if_fail (PHOSH_IS_NETWORK_AUTH_PROMPT (self));
 
@@ -353,7 +354,7 @@ phosh_network_auth_prompt_class_init (PhoshNetworkAuthPromptClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PhoshNetworkAuthPrompt, wpa_password_entry);
   gtk_widget_class_bind_template_child (widget_class, PhoshNetworkAuthPrompt, password_buffer);
 
-  gtk_widget_class_bind_template_callback (widget_class, network_prompt_cancel_clicked_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_dialog_canceled);
   gtk_widget_class_bind_template_callback (widget_class, network_prompt_connect_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, network_prompt_key_press_event_cb);
   gtk_widget_class_bind_template_callback (widget_class, network_prompt_wpa_password_changed_cb);
@@ -370,28 +371,14 @@ phosh_network_auth_prompt_init (PhoshNetworkAuthPrompt *self)
 
 GtkWidget *
 phosh_network_auth_prompt_new (ShellNetworkAgent *agent,
-                               NMClient          *nm_client,
-                               gpointer           layer_shell,
-                               gpointer           wl_output)
+                               NMClient          *nm_client)
 {
   PhoshNetworkAuthPrompt *self;
 
   g_return_val_if_fail (SHELL_IS_NETWORK_AGENT (agent), NULL);
   g_return_val_if_fail (NM_CLIENT (nm_client), NULL);
 
-  self = g_object_new (PHOSH_TYPE_NETWORK_AUTH_PROMPT,
-                       /* layer shell */
-                       "layer-shell", layer_shell,
-                       "wl-output", wl_output,
-                       "anchor", ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP |
-                       ZWLR_LAYER_SURFACE_V1_ANCHOR_BOTTOM |
-                       ZWLR_LAYER_SURFACE_V1_ANCHOR_LEFT |
-                       ZWLR_LAYER_SURFACE_V1_ANCHOR_RIGHT,
-                       "layer", ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
-                       "kbd-interactivity", TRUE,
-                       "exclusive-zone", -1,
-                       "namespace", "phosh prompter",
-                       NULL);
+  self = g_object_new (PHOSH_TYPE_NETWORK_AUTH_PROMPT, NULL);
   self->nm_client = g_object_ref (nm_client);
   self->agent = g_object_ref (agent);
 
