@@ -7,18 +7,16 @@
 
 #include <gtk/gtk.h>
 #include <NetworkManager.h>
-#include "layersurface.h"
+#include "system-modal-dialog.h"
 
 G_BEGIN_DECLS
 
 #define PHOSH_TYPE_NETWORK_AUTH_PROMPT (phosh_network_auth_prompt_get_type())
 
-G_DECLARE_FINAL_TYPE (PhoshNetworkAuthPrompt, phosh_network_auth_prompt, PHOSH, NETWORK_AUTH_PROMPT, PhoshLayerSurface);
+G_DECLARE_FINAL_TYPE (PhoshNetworkAuthPrompt, phosh_network_auth_prompt, PHOSH, NETWORK_AUTH_PROMPT, PhoshSystemModalDialog);
 
 GtkWidget *phosh_network_auth_prompt_new         (ShellNetworkAgent *agent,
-                                                  NMClient          *nm_client,
-                                                  gpointer           layer_shell,
-                                                  gpointer           wl_output);
+                                                  NMClient          *nm_client);
 void       phosh_network_auth_prompt_set_request (PhoshNetworkAuthPrompt        *self,
                                                   char                          *request_id,
                                                   NMConnection                  *connection,
