@@ -16,6 +16,7 @@
 #include "notification-banner.h"
 #include "notification-list.h"
 #include "notify-manager.h"
+#include "notify-feedback.h"
 #include "shell.h"
 #include "phosh-enums.h"
 #include "util.h"
@@ -40,6 +41,8 @@
  * #PhoshNotifyManager manages notifications sent from the shell
  * iself and via the org.freedesktop.Notification DBus interface.
  * See https://developer.gnome.org/notification-spec/
+ *
+ * It maintains a list of notifications via a #PhoshNotificationList.
  */
 
 #define NOTIFY_DBUS_NAME "org.freedesktop.Notifications"
@@ -59,7 +62,14 @@ typedef struct _PhoshNotifyManager
 
   GSettings *settings;
 
+  /* Notification to be handled on unlock */
+  struct {
+    PhoshNotification *notification;
+    char              *action;
+  } unlock_notify;
+
   PhoshNotificationList *list;
+  PhoshNotifyFeedback *feedback;
 } PhoshNotifyManager;
 
 G_DEFINE_TYPE_WITH_CODE (PhoshNotifyManager,
@@ -158,19 +168,39 @@ on_notification_expired (PhoshNotifyManager *self,
 
 
 static void
-on_notification_actioned (PhoshNotifyManager *self,
-                          const char         *action,
-                          PhoshNotification  *notification)
+on_unlock_notify_ref_gone (gpointer data, GObject *gone)
 {
-  guint id;
+  PhoshNotifyManager *self = PHOSH_NOTIFY_MANAGER (data);
 
   g_return_if_fail (PHOSH_IS_NOTIFY_MANAGER (self));
-  g_return_if_fail (PHOSH_IS_NOTIFICATION (notification));
+
+  self->unlock_notify.notification = NULL;
+  g_clear_pointer (&self->unlock_notify.action, g_free);
+}
+
+
+static void
+forget_unlock_notify (PhoshNotifyManager *self)
+{
+  if (!self->unlock_notify.notification)
+    return;
+
+  g_object_weak_unref (G_OBJECT (self->unlock_notify.notification),
+                       on_unlock_notify_ref_gone,
+                       self);
+  self->unlock_notify.notification = NULL;
+  g_clear_pointer (&self->unlock_notify.action, g_free);
+}
+
+
+static void
+invoke_action (PhoshNotification *notification, const gchar *action)
+{
+  guint id;
 
   id = phosh_notification_get_id (notification);
 
   g_return_if_fail (id);
-
   g_debug ("Emitting ActionInvoked: %d, %s", id, action);
 
   phosh_notification_do_action (notification, id, action);
@@ -183,6 +213,55 @@ on_notification_actioned (PhoshNotifyManager *self,
 }
 
 
+
+static void
+on_shell_lock_changed (PhoshNotifyManager* self, GParamSpec *pspec, PhoshShell *shell)
+{
+  gboolean locked;
+
+  g_return_if_fail (PHOSH_IS_NOTIFY_MANAGER (self));
+
+  locked = phosh_shell_get_locked (shell);
+  if (!locked && self->unlock_notify.notification) {
+    invoke_action (self->unlock_notify.notification, self->unlock_notify.action);
+    forget_unlock_notify (self);
+  }
+}
+
+
+static void
+on_notification_actioned (PhoshNotifyManager *self,
+                          const char         *action,
+                          PhoshNotification  *notification)
+{
+  PhoshShell *shell = phosh_shell_get_default();
+
+  g_return_if_fail (PHOSH_IS_NOTIFY_MANAGER (self));
+  g_return_if_fail (PHOSH_IS_NOTIFICATION (notification));
+
+  /* Postpone action when shell is locked */
+  if (phosh_shell_get_locked (shell)) {
+    PhoshLockscreenManager *lm;
+
+    /* Forget any pending actions */
+    forget_unlock_notify (self);
+
+    /* Clear out notification if it goes away in the meantime */
+    g_object_weak_ref (G_OBJECT (notification),
+                       on_unlock_notify_ref_gone,
+                       self);
+    self->unlock_notify.notification = notification;
+    self->unlock_notify.action = g_strdup (action);
+
+    /* Scroll to unlock page */
+    lm = phosh_shell_get_lockscreen_manager (shell);
+    phosh_lockscreen_manager_set_page (lm, PHOSH_LOCKSCREEN_PAGE_UNLOCK);
+  } else {
+    invoke_action (notification, action);
+  }
+}
+
+
 static void
 on_notification_closed (PhoshNotifyManager      *self,
                         PhoshNotificationReason  reason,
@@ -548,8 +627,13 @@ phosh_notify_manager_dispose (GObject *object)
 {
   PhoshNotifyManager *self = PHOSH_NOTIFY_MANAGER (object);
 
-  g_clear_object (&self->settings);
+  g_clear_handle_id (&self->dbus_name_id, g_bus_unown_name);
 
+  if (g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (self)))
+    g_dbus_interface_skeleton_unexport (G_DBUS_INTERFACE_SKELETON (self));
+
+  g_clear_object (&self->settings);
+  g_clear_object (&self->feedback);
   g_clear_object (&self->list);
 
   G_OBJECT_CLASS (phosh_notify_manager_parent_class)->dispose (object);
@@ -572,6 +656,7 @@ static void
 phosh_notify_manager_constructed (GObject *object)
 {
   PhoshNotifyManager *self = PHOSH_NOTIFY_MANAGER (object);
+  PhoshShell *shell = phosh_shell_get_default();
 
   G_OBJECT_CLASS (phosh_notify_manager_parent_class)->constructed (object);
   self->dbus_name_id = g_bus_own_name (G_BUS_TYPE_SESSION,
@@ -581,8 +666,8 @@ phosh_notify_manager_constructed (GObject *object)
                                        on_bus_acquired,
                                        on_name_acquired,
                                        on_name_lost,
-                                       g_object_ref (self),
-                                       g_object_unref);
+                                       self,
+                                       NULL);
 
   self->settings = g_settings_new (NOTIFICATIONS_SCHEMA_ID);
   g_signal_connect_swapped (self->settings, "changed::" NOTIFICATIONS_KEY_SHOW_BANNERS,
@@ -592,6 +677,10 @@ phosh_notify_manager_constructed (GObject *object)
   g_signal_connect_swapped (self->settings, "changed::" NOTIFICATIONS_KEY_APP_CHILDREN,
                             G_CALLBACK (on_notification_apps_setting_changed), self);
   on_notification_apps_setting_changed (self, NULL, self->settings);
+
+  g_signal_connect_swapped (shell, "notify::locked", G_CALLBACK (on_shell_lock_changed), self);
+
+  self->feedback = phosh_notify_feedback_new (self->list);
 }
 
 
@@ -705,8 +794,7 @@ phosh_notify_manager_get_notification_id (PhoshNotifyManager *self)
  * @expire_timeout: When the notification should expire
  * @notification: The notification
  *
- * Returns: Adds a notification
- * notifications.
+ * Adds @notification to the current list of notifications.
  */
 void
 phosh_notify_manager_add_notification (PhoshNotifyManager *self,
@@ -803,3 +891,28 @@ phosh_notify_manager_get_show_notification_banner (PhoshNotifyManager *self,
   g_debug ("Show banners for %s: %d", munged_id, show);
   return show;
 }
+
+/**
+ * phosh_notify_manager_close_all:
+ * @self: the #PhoshNotifyManager
+ * @rease: the #PhoshNotificationReason
+ *
+ * Closes all notifications using #PhoshNotificationReason as reason.
+ */
+void
+phosh_notify_manager_close_all_notifications (PhoshNotifyManager      *self,
+                                              PhoshNotificationReason  reason)
+{
+  GListModel *source_list;
+
+  source_list = G_LIST_MODEL (phosh_notify_manager_get_list (self));
+
+  for (int i = g_list_model_get_n_items(G_LIST_MODEL (source_list)); i > 0; i--) {
+    g_autoptr (GListModel) notif_list = G_LIST_MODEL (g_list_model_get_object (source_list, i-1));
+
+    for (int j = g_list_model_get_n_items(G_LIST_MODEL (notif_list)); j > 0; j--) {
+      g_autoptr (PhoshNotification) notification = PHOSH_NOTIFICATION (g_list_model_get_object (notif_list, j-1));
+      phosh_notification_close (notification, PHOSH_NOTIFICATION_REASON_DISMISSED);
+    }
+  }
+}
