@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2020 Purism SPC
  *
- * SPDX-License-Identifier: GPL-3.0+
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #pragma once
@@ -23,6 +23,6 @@ struct _PhoshQuickSettingClass
 GtkWidget * phosh_quick_setting_new (void);
 void phosh_quick_setting_set_status_icon (PhoshQuickSetting *self, PhoshStatusIcon *widget);
 PhoshStatusIcon * phosh_quick_setting_get_status_icon (PhoshQuickSetting *self);
-void phosh_quick_setting_open_settings_panel (gchar *panel);
+void phosh_quick_setting_open_settings_panel (char *panel);
 
 G_END_DECLS
