@@ -30,12 +30,14 @@
 #include "bt-info.h"
 #include "bt-manager.h"
 #include "connectivity-info.h"
+#include "calls-manager.h"
 #include "docked-info.h"
 #include "docked-manager.h"
 #include "fader.h"
 #include "feedbackinfo.h"
 #include "feedback-manager.h"
 #include "gnome-shell-manager.h"
+#include "gtk-mount-manager.h"
 #include "hks-info.h"
 #include "home.h"
 #include "idle-manager.h"
@@ -53,7 +55,6 @@
 #include "notifications/notify-manager.h"
 #include "notifications/notification-banner.h"
 #include "osk-manager.h"
-#include "panel.h"
 #include "phosh-wayland.h"
 #include "polkit-auth-agent.h"
 #include "proximity.h"
@@ -62,8 +63,10 @@
 #include "rotation-manager.h"
 #include "sensor-proxy-manager.h"
 #include "screen-saver-manager.h"
+#include "screenshot-manager.h"
 #include "session-manager.h"
 #include "system-prompter.h"
+#include "top-panel.h"
 #include "torch-manager.h"
 #include "torch-info.h"
 #include "util.h"
@@ -79,20 +82,26 @@
  * @Title: PhoshShell
  *
  * #PhoshShell is responsible for instantiating the GUI
- * parts of the shell#PhoshPanel, #PhoshHome,… and the managers that
+ * parts of the shell#PhoshTopPanel, #PhoshHome,… and the managers that
  * interface with DBus #PhoshMonitorManager, #PhoshFeedbackManager, …
  * and coordinates between them.
  */
 
 enum {
-  PHOSH_SHELL_PROP_0,
-  PHOSH_SHELL_PROP_LOCKED,
-  PHOSH_SHELL_PROP_BUILTIN_MONITOR,
-  PHOSH_SHELL_PROP_PRIMARY_MONITOR,
-  PHOSH_SHELL_PROP_SHELL_STATE,
-  PHOSH_SHELL_PROP_LAST_PROP
+  PROP_0,
+  PROP_LOCKED,
+  PROP_BUILTIN_MONITOR,
+  PROP_PRIMARY_MONITOR,
+  PROP_SHELL_STATE,
+  PROP_LAST_PROP
 };
-static GParamSpec *props[PHOSH_SHELL_PROP_LAST_PROP];
+static GParamSpec *props[PROP_LAST_PROP];
+
+enum {
+  READY,
+  N_SIGNALS
+};
+static guint signals[N_SIGNALS] = { 0 };
 
 typedef struct
 {
@@ -104,6 +113,7 @@ typedef struct
 
   PhoshSessionManager *session_manager;
   PhoshBackgroundManager *background_manager;
+  PhoshCallsManager *calls_manager;
   PhoshMonitor *primary_monitor;
   PhoshMonitor *builtin_monitor;
   PhoshMonitorManager *monitor_manager;
@@ -114,6 +124,7 @@ typedef struct
   PhoshWifiManager *wifi_manager;
   PhoshPolkitAuthAgent *polkit_auth_agent;
   PhoshScreenSaverManager *screen_saver_manager;
+  PhoshScreenshotManager *screenshot_manager;
   PhoshNotifyManager *notify_manager;
   PhoshFeedbackManager *feedback_manager;
   PhoshBtManager *bt_manager;
@@ -122,6 +133,7 @@ typedef struct
   PhoshTorchManager *torch_manager;
   PhoshModeManager *mode_manager;
   PhoshDockedManager *docked_manager;
+  PhoshGtkMountManager *gtk_mount_manager;
   PhoshHksManager *hks_manager;
   PhoshKeyboardEvents *keyboard_events;
   PhoshLocationManager *location_manager;
@@ -139,6 +151,8 @@ typedef struct
 
   PhoshShellStateFlags shell_state;
 
+  char           *theme_name;
+  GtkCssProvider *css_provider;
 } PhoshShellPrivate;
 
 
@@ -151,63 +165,16 @@ G_DEFINE_TYPE_WITH_PRIVATE (PhoshShell, phosh_shell, G_TYPE_OBJECT)
 
 
 static void
-settings_activated_cb (PhoshShell *self,
-                       PhoshPanel *window)
+settings_activated_cb (PhoshShell    *self,
+                       PhoshTopPanel *window)
 {
   PhoshShellPrivate *priv = phosh_shell_get_instance_private (self);
 
-  g_return_if_fail (PHOSH_IS_PANEL (priv->panel));
-  phosh_panel_toggle_fold (PHOSH_PANEL(priv->panel));
+  g_return_if_fail (PHOSH_IS_TOP_PANEL (priv->panel));
+  phosh_top_panel_toggle_fold (PHOSH_TOP_PANEL(priv->panel));
 }
 
 
-void
-phosh_shell_lock (PhoshShell *self)
-{
-  phosh_shell_set_locked (self, TRUE);
-}
-
-
-void
-phosh_shell_unlock (PhoshShell *self)
-{
-  phosh_shell_set_locked (self, FALSE);
-}
-
-/**
- * phosh_shell_get_locked:
- * @self: The #PhoshShell singleton
- *
- * Returns: %TRUE if the shell is currently locked, otherwise %FALSE.
- */
-gboolean
-phosh_shell_get_locked (PhoshShell *self)
-{
-  PhoshShellPrivate *priv = phosh_shell_get_instance_private (self);
-
-  return priv->locked;
-}
-
-/**
- * phosh_shell_set_locked:
- * @self: The #PhoshShell singleton
- * @locked: %TRUE to lock the shell
- *
- * Lock the shell. We proxy to lockscreen-manager to avoid
- * that other parts of the shell need to care about this
- * abstraction.
- */
-void
-phosh_shell_set_locked (PhoshShell *self, gboolean locked)
-{
-  PhoshShellPrivate *priv = phosh_shell_get_instance_private (self);
-
-  if (locked == priv->locked)
-    return;
-
-  phosh_lockscreen_manager_set_locked (priv->lockscreen_manager, locked);
-}
-
 static void
 on_home_state_changed (PhoshShell *self, GParamSpec *pspec, PhoshHome *home)
 {
@@ -221,7 +188,7 @@ on_home_state_changed (PhoshShell *self, GParamSpec *pspec, PhoshHome *home)
 
   g_object_get (priv->home, "state", &state, NULL);
   if (state == PHOSH_HOME_STATE_UNFOLDED) {
-    phosh_panel_fold (PHOSH_PANEL (priv->panel));
+    phosh_top_panel_fold (PHOSH_TOP_PANEL (priv->panel));
     phosh_osk_manager_set_visible (priv->osk_manager, FALSE);
   }
   phosh_shell_set_state (self, PHOSH_STATE_OVERVIEW, state == PHOSH_HOME_STATE_UNFOLDED);
@@ -234,11 +201,12 @@ panels_create (PhoshShell *self)
   PhoshShellPrivate *priv = phosh_shell_get_instance_private (self);
   PhoshMonitor *monitor;
   PhoshWayland *wl = phosh_wayland_get_default ();
+  PhoshAppGrid *app_grid;
 
   monitor = phosh_shell_get_primary_monitor (self);
   g_return_if_fail (monitor);
 
-  priv->panel = PHOSH_LAYER_SURFACE(phosh_panel_new (phosh_wayland_get_zwlr_layer_shell_v1(wl),
+  priv->panel = PHOSH_LAYER_SURFACE(phosh_top_panel_new (phosh_wayland_get_zwlr_layer_shell_v1(wl),
                                                      monitor->wl_output));
   gtk_widget_show (GTK_WIDGET (priv->panel));
 
@@ -257,6 +225,13 @@ panels_create (PhoshShell *self)
     "notify::state",
     G_CALLBACK(on_home_state_changed),
     self);
+
+  app_grid = phosh_overview_get_app_grid (phosh_home_get_overview (PHOSH_HOME (priv->home)));
+  g_object_bind_property (priv->docked_manager,
+                          "enabled",
+                          app_grid,
+                          "filter-adaptive",
+                          G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
 }
 
 
@@ -270,15 +245,38 @@ panels_dispose (PhoshShell *self)
 }
 
 
+/* Select proper style sheet in case of high contrast */
 static void
-css_setup (PhoshShell *self)
+on_gtk_theme_name_changed (PhoshShell *self, GParamSpec *pspec, GtkSettings *settings)
 {
+  const char *style;
+  g_autofree char *name = NULL;
+  PhoshShellPrivate *priv = phosh_shell_get_instance_private (self);
   g_autoptr (GtkCssProvider) provider = gtk_css_provider_new ();
 
-  gtk_css_provider_load_from_resource (provider, "/sm/puri/phosh/style.css");
+  g_object_get (settings, "gtk-theme-name", &name, NULL);
+
+  if (g_strcmp0 (priv->theme_name, name) == 0)
+    return;
+
+  priv->theme_name = g_steal_pointer (&name);
+  g_debug ("GTK theme: %s", priv->theme_name);
+
+  if (priv->css_provider) {
+    gtk_style_context_remove_provider_for_screen(gdk_screen_get_default (),
+                                                 GTK_STYLE_PROVIDER (priv->css_provider));
+  }
+
+  if (g_strcmp0 (priv->theme_name, "HighContrast") == 0)
+    style = "/sm/puri/phosh/stylesheet/adwaita-hc-light.css";
+  else
+    style = "/sm/puri/phosh/stylesheet/adwaita-dark.css";
+
+  gtk_css_provider_load_from_resource (provider, style);
   gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
                                              GTK_STYLE_PROVIDER (provider),
                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_set_object (&priv->css_provider, provider);
 }
 
 
@@ -292,11 +290,11 @@ phosh_shell_set_property (GObject *object,
   PhoshShellPrivate *priv = phosh_shell_get_instance_private(self);
 
   switch (property_id) {
-  case PHOSH_SHELL_PROP_LOCKED:
+  case PROP_LOCKED:
     priv->locked = g_value_get_boolean (value);
     phosh_shell_set_state (self, PHOSH_STATE_LOCKED, priv->locked);
     break;
-  case PHOSH_SHELL_PROP_PRIMARY_MONITOR:
+  case PROP_PRIMARY_MONITOR:
     phosh_shell_set_primary_monitor (self, g_value_get_object (value));
     break;
   default:
@@ -316,16 +314,16 @@ phosh_shell_get_property (GObject *object,
   PhoshShellPrivate *priv = phosh_shell_get_instance_private(self);
 
   switch (property_id) {
-  case PHOSH_SHELL_PROP_LOCKED:
+  case PROP_LOCKED:
     g_value_set_boolean (value, priv->locked);
     break;
-  case PHOSH_SHELL_PROP_BUILTIN_MONITOR:
+  case PROP_BUILTIN_MONITOR:
     g_value_set_object (value, phosh_shell_get_builtin_monitor (self));
     break;
-  case PHOSH_SHELL_PROP_PRIMARY_MONITOR:
+  case PROP_PRIMARY_MONITOR:
     g_value_set_object (value, phosh_shell_get_primary_monitor (self));
     break;
-  case PHOSH_SHELL_PROP_SHELL_STATE:
+  case PROP_SHELL_STATE:
     g_value_set_flags (value, priv->shell_state);
     break;
   default:
@@ -346,10 +344,12 @@ phosh_shell_dispose (GObject *object)
 
   g_clear_object (&priv->notification_banner);
 
-  g_clear_object (&priv->keyboard_events);
   /* dispose managers in opposite order of declaration */
+  g_clear_object (&priv->screenshot_manager);
+  g_clear_object (&priv->calls_manager);
   g_clear_object (&priv->location_manager);
   g_clear_object (&priv->hks_manager);
+  g_clear_object (&priv->gtk_mount_manager);
   g_clear_object (&priv->docked_manager);
   g_clear_object (&priv->mode_manager);
   g_clear_object (&priv->torch_manager);
@@ -370,6 +370,7 @@ phosh_shell_dispose (GObject *object)
   g_clear_object (&priv->builtin_monitor);
   g_clear_object (&priv->primary_monitor);
   g_clear_object (&priv->background_manager);
+  g_clear_object (&priv->keyboard_events);
 
   /* sensors */
   g_clear_object (&priv->proximity);
@@ -379,6 +380,9 @@ phosh_shell_dispose (GObject *object)
   phosh_system_prompter_unregister ();
   g_clear_object (&priv->session_manager);
 
+  g_clear_pointer (&priv->theme_name, g_free);
+  g_clear_object (&priv->css_provider);
+
   G_OBJECT_CLASS (phosh_shell_parent_class)->dispose (object);
 }
 
@@ -431,7 +435,7 @@ on_new_notification (PhoshShell         *self,
   }
 
   if (phosh_notify_manager_get_show_notification_banner (manager, notification) &&
-      phosh_panel_get_state (PHOSH_PANEL (priv->panel)) == PHOSH_PANEL_STATE_FOLDED &&
+      phosh_top_panel_get_state (PHOSH_TOP_PANEL (priv->panel)) == PHOSH_TOP_PANEL_STATE_FOLDED &&
       !priv->locked) {
     g_set_weak_pointer (&priv->notification_banner,
                         phosh_notification_banner_new (notification));
@@ -468,7 +472,7 @@ setup_idle_cb (PhoshShell *self)
 
   priv->sensor_proxy_manager = phosh_sensor_proxy_manager_new (&err);
   if (!priv->sensor_proxy_manager)
-    g_warning ("Failed to connect to sensor-proxy: %s", err->message);
+    g_message ("Failed to connect to sensor-proxy: %s", err->message);
 
   panels_create (self);
   /* Create background after panel since it needs the panel's size */
@@ -501,12 +505,13 @@ setup_idle_cb (PhoshShell *self)
   phosh_shell_get_location_manager (self);
   if (priv->sensor_proxy_manager) {
     priv->proximity = phosh_proximity_new (priv->sensor_proxy_manager,
-                                           priv->lockscreen_manager);
+                                           priv->calls_manager);
     phosh_monitor_manager_set_sensor_proxy_manager (priv->monitor_manager,
                                                     priv->sensor_proxy_manager);
   }
 
   priv->mount_manager = phosh_mount_manager_new ();
+  priv->gtk_mount_manager = phosh_gtk_mount_manager_new ();
 
   phosh_session_manager_register (priv->session_manager,
                                   PHOSH_APP_ID,
@@ -514,8 +519,10 @@ setup_idle_cb (PhoshShell *self)
   g_unsetenv ("DESKTOP_AUTOSTART_ID");
 
   priv->gnome_shell_manager = phosh_gnome_shell_manager_get_default ();
+  priv->screenshot_manager = phosh_screenshot_manager_new ();
 
   priv->startup_finished = TRUE;
+  g_signal_emit (self, signals[READY], 0);
 
   return FALSE;
 }
@@ -548,12 +555,15 @@ static void
 on_builtin_monitor_power_mode_changed (PhoshShell *self, GParamSpec *pspec, PhoshMonitor *monitor)
 {
   PhoshMonitorPowerSaveMode mode;
+  PhoshShellPrivate *priv;
 
   g_return_if_fail (PHOSH_IS_SHELL (self));
   g_return_if_fail (PHOSH_IS_MONITOR (monitor));
+  priv = phosh_shell_get_instance_private (self);
 
   g_object_get (monitor, "power-mode", &mode, NULL);
-  if (mode == PHOSH_MONITOR_POWER_SAVE_MODE_OFF)
+  /* Might be emitted on startup before lockscreen_manager is up */
+  if (mode == PHOSH_MONITOR_POWER_SAVE_MODE_OFF && priv->lockscreen_manager)
     phosh_shell_lock (self);
 
   phosh_shell_set_state (self, PHOSH_STATE_BLANKED, mode == PHOSH_MONITOR_POWER_SAVE_MODE_OFF);
@@ -586,7 +596,7 @@ on_monitor_added (PhoshShell *self, PhoshMonitor *monitor)
   if (priv->rotation_manager)
     phosh_rotation_manager_set_monitor (priv->rotation_manager, monitor);
 
-  g_object_notify_by_pspec (G_OBJECT (self), props[PHOSH_SHELL_PROP_BUILTIN_MONITOR]);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_BUILTIN_MONITOR]);
 }
 
 
@@ -611,7 +621,7 @@ on_monitor_removed (PhoshShell *self, PhoshMonitor *monitor)
     if (priv->rotation_manager)
       phosh_rotation_manager_set_monitor (priv->rotation_manager, NULL);
 
-    g_object_notify_by_pspec (G_OBJECT (self), props[PHOSH_SHELL_PROP_BUILTIN_MONITOR]);
+    g_object_notify_by_pspec (G_OBJECT (self), props[PROP_BUILTIN_MONITOR]);
   }
 
   if (priv->primary_monitor == monitor) {
@@ -710,9 +720,9 @@ phosh_shell_constructed (GObject *object)
 
   gtk_icon_theme_add_resource_path (gtk_icon_theme_get_default (),
                                     "/sm/puri/phosh/icons");
-  css_setup (self);
 
-  priv->lockscreen_manager = phosh_lockscreen_manager_new ();
+  priv->calls_manager = phosh_calls_manager_new ();
+  priv->lockscreen_manager = phosh_lockscreen_manager_new (priv->calls_manager);
   g_object_bind_property (priv->lockscreen_manager, "locked",
                           self, "locked",
                           G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
@@ -744,7 +754,7 @@ phosh_shell_class_init (PhoshShellClass *klass)
 
   type_setup ();
 
-  props[PHOSH_SHELL_PROP_LOCKED] =
+  props[PROP_LOCKED] =
     g_param_spec_boolean ("locked",
                           "Locked",
                           "Whether the screen is locked",
@@ -757,7 +767,7 @@ phosh_shell_class_init (PhoshShellClass *klass)
    * The built in monitor. This is a hardware property and hence can
    * only be read. It can be %NULL when not present or disabled.
    */
-  props[PHOSH_SHELL_PROP_BUILTIN_MONITOR] =
+  props[PROP_BUILTIN_MONITOR] =
     g_param_spec_object ("builtin-monitor",
                          "Built in monitor",
                          "The builtin monitor",
@@ -768,14 +778,14 @@ phosh_shell_class_init (PhoshShellClass *klass)
    *
    * The primary monitor that has the panels, lock screen etc. This can't be %NULL.
    */
-  props[PHOSH_SHELL_PROP_PRIMARY_MONITOR] =
+  props[PROP_PRIMARY_MONITOR] =
     g_param_spec_object ("primary-monitor",
                          "Primary monitor",
                          "The primary monitor",
                          PHOSH_TYPE_MONITOR,
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
-  props[PHOSH_SHELL_PROP_SHELL_STATE] =
+  props[PROP_SHELL_STATE] =
     g_param_spec_flags ("shell-state",
                         "Shell state",
                         "The state of the shell",
@@ -783,7 +793,13 @@ phosh_shell_class_init (PhoshShellClass *klass)
                         PHOSH_STATE_NONE,
                         G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
-  g_object_class_install_properties (object_class, PHOSH_SHELL_PROP_LAST_PROP, props);
+  g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
+
+  signals[READY] = g_signal_new ("ready",
+                                 G_TYPE_FROM_CLASS (klass),
+                                 G_SIGNAL_RUN_LAST, 0,
+                                 NULL, NULL, NULL,
+                                 G_TYPE_NONE, 0);
 }
 
 
@@ -796,6 +812,9 @@ phosh_shell_init (PhoshShell *self)
   gtk_settings = gtk_settings_get_default ();
   g_object_set (G_OBJECT (gtk_settings), "gtk-application-prefer-dark-theme", TRUE, NULL);
 
+  g_signal_connect_swapped (gtk_settings, "notify::gtk-theme-name", G_CALLBACK (on_gtk_theme_name_changed), self);
+  on_gtk_theme_name_changed (self, NULL, gtk_settings);
+
   priv->shell_state = PHOSH_STATE_NONE;
 }
 
@@ -828,7 +847,7 @@ phosh_shell_set_primary_monitor (PhoshShell *self, PhoshMonitor *monitor)
   panels_dispose (self);
   panels_create (self);
 
-  g_object_notify_by_pspec (G_OBJECT (self), props[PHOSH_SHELL_PROP_PRIMARY_MONITOR]);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_PRIMARY_MONITOR]);
 }
 
 
@@ -864,6 +883,59 @@ phosh_shell_get_primary_monitor (PhoshShell *self)
   return monitor;
 }
 
+/* Manager getters */
+
+PhoshBackgroundManager *
+phosh_shell_get_background_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+  g_return_val_if_fail (PHOSH_IS_BACKGROUND_MANAGER (priv->background_manager), NULL);
+
+  return priv->background_manager;
+}
+
+
+PhoshCallsManager *
+phosh_shell_get_calls_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+  g_return_val_if_fail (PHOSH_IS_CALLS_MANAGER (priv->calls_manager), NULL);
+
+  return priv->calls_manager;
+}
+
+
+PhoshFeedbackManager *
+phosh_shell_get_feedback_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+  g_return_val_if_fail (PHOSH_IS_FEEDBACK_MANAGER (priv->feedback_manager), NULL);
+
+  return priv->feedback_manager;
+}
+
+
+PhoshGtkMountManager *
+phosh_shell_get_gtk_mount_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+  g_return_val_if_fail (PHOSH_IS_GTK_MOUNT_MANAGER (priv->gtk_mount_manager), NULL);
+
+  return priv->gtk_mount_manager;
+}
+
 
 PhoshLockscreenManager *
 phosh_shell_get_lockscreen_manager (PhoshShell *self)
@@ -904,22 +976,33 @@ phosh_shell_get_monitor_manager (PhoshShell *self)
 }
 
 
-PhoshWifiManager *
-phosh_shell_get_wifi_manager (PhoshShell *self)
+PhoshToplevelManager *
+phosh_shell_get_toplevel_manager (PhoshShell *self)
 {
   PhoshShellPrivate *priv;
 
   g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
   priv = phosh_shell_get_instance_private (self);
 
-  if (!priv->wifi_manager)
-      priv->wifi_manager = phosh_wifi_manager_new ();
-
-  g_return_val_if_fail (PHOSH_IS_WIFI_MANAGER (priv->wifi_manager), NULL);
-  return priv->wifi_manager;
+  g_return_val_if_fail (PHOSH_IS_TOPLEVEL_MANAGER (priv->toplevel_manager), NULL);
+  return priv->toplevel_manager;
 }
 
 
+PhoshSessionManager *
+phosh_shell_get_session_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+  g_return_val_if_fail (PHOSH_IS_SESSION_MANAGER (priv->session_manager), NULL);
+
+  return priv->session_manager;
+}
+
+/* Manager getters that create them as needed */
+
 PhoshBtManager *
 phosh_shell_get_bt_manager (PhoshShell *self)
 {
@@ -936,92 +1019,6 @@ phosh_shell_get_bt_manager (PhoshShell *self)
 }
 
 
-PhoshOskManager *
-phosh_shell_get_osk_manager (PhoshShell *self)
-{
-  PhoshShellPrivate *priv;
-
-  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
-  priv = phosh_shell_get_instance_private (self);
-
-  if (!priv->osk_manager)
-      priv->osk_manager = phosh_osk_manager_new ();
-
-  g_return_val_if_fail (PHOSH_IS_OSK_MANAGER (priv->osk_manager), NULL);
-  return priv->osk_manager;
-}
-
-
-PhoshToplevelManager *
-phosh_shell_get_toplevel_manager (PhoshShell *self)
-{
-  PhoshShellPrivate *priv;
-
-  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
-  priv = phosh_shell_get_instance_private (self);
-
-  g_return_val_if_fail (PHOSH_IS_TOPLEVEL_MANAGER (priv->toplevel_manager), NULL);
-  return priv->toplevel_manager;
-}
-
-
-PhoshFeedbackManager *
-phosh_shell_get_feedback_manager (PhoshShell *self)
-{
-  PhoshShellPrivate *priv;
-
-  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
-  priv = phosh_shell_get_instance_private (self);
-  g_return_val_if_fail (PHOSH_IS_FEEDBACK_MANAGER (priv->feedback_manager), NULL);
-
-  return priv->feedback_manager;
-}
-
-
-PhoshWWan *
-phosh_shell_get_wwan (PhoshShell *self)
-{
-  PhoshShellPrivate *priv;
-
-  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
-  priv = phosh_shell_get_instance_private (self);
-
-  if (!priv->wwan) {
-    g_autoptr (GSettings) settings = g_settings_new ("sm.puri.phosh");
-    PhoshWWanBackend backend = g_settings_get_enum (settings, WWAN_BACKEND_KEY);
-
-    switch (backend) {
-      default:
-      case PHOSH_WWAN_BACKEND_MM:
-        priv->wwan = PHOSH_WWAN (phosh_wwan_mm_new());
-        break;
-      case PHOSH_WWAN_BACKEND_OFONO:
-        priv->wwan = PHOSH_WWAN (phosh_wwan_ofono_new());
-        break;
-    }
-  }
-
-  g_return_val_if_fail (PHOSH_IS_WWAN (priv->wwan), NULL);
-  return priv->wwan;
-}
-
-
-PhoshTorchManager *
-phosh_shell_get_torch_manager (PhoshShell *self)
-{
-  PhoshShellPrivate *priv;
-
-  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
-  priv = phosh_shell_get_instance_private (self);
-
-  if (!priv->torch_manager)
-    priv->torch_manager = phosh_torch_manager_new ();
-
-  g_return_val_if_fail (PHOSH_IS_TORCH_MANAGER (priv->torch_manager), NULL);
-  return priv->torch_manager;
-}
-
-
 PhoshDockedManager *
 phosh_shell_get_docked_manager (PhoshShell *self)
 {
@@ -1070,16 +1067,19 @@ phosh_shell_get_location_manager (PhoshShell *self)
 }
 
 
-PhoshSessionManager *
-phosh_shell_get_session_manager (PhoshShell *self)
+PhoshOskManager *
+phosh_shell_get_osk_manager (PhoshShell *self)
 {
   PhoshShellPrivate *priv;
 
   g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
   priv = phosh_shell_get_instance_private (self);
-  g_return_val_if_fail (PHOSH_IS_SESSION_MANAGER (priv->session_manager), NULL);
 
-  return priv->session_manager;
+  if (!priv->osk_manager)
+      priv->osk_manager = phosh_osk_manager_new ();
+
+  g_return_val_if_fail (PHOSH_IS_OSK_MANAGER (priv->osk_manager), NULL);
+  return priv->osk_manager;
 }
 
 
@@ -1101,6 +1101,66 @@ phosh_shell_get_rotation_manager (PhoshShell *self)
   return priv->rotation_manager;
 }
 
+
+PhoshTorchManager *
+phosh_shell_get_torch_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+
+  if (!priv->torch_manager)
+    priv->torch_manager = phosh_torch_manager_new ();
+
+  g_return_val_if_fail (PHOSH_IS_TORCH_MANAGER (priv->torch_manager), NULL);
+  return priv->torch_manager;
+}
+
+
+PhoshWifiManager *
+phosh_shell_get_wifi_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+
+  if (!priv->wifi_manager)
+      priv->wifi_manager = phosh_wifi_manager_new ();
+
+  g_return_val_if_fail (PHOSH_IS_WIFI_MANAGER (priv->wifi_manager), NULL);
+  return priv->wifi_manager;
+}
+
+
+PhoshWWan *
+phosh_shell_get_wwan (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+
+  if (!priv->wwan) {
+    g_autoptr (GSettings) settings = g_settings_new ("sm.puri.phosh");
+    PhoshWWanBackend backend = g_settings_get_enum (settings, WWAN_BACKEND_KEY);
+
+    switch (backend) {
+      default:
+      case PHOSH_WWAN_BACKEND_MM:
+        priv->wwan = PHOSH_WWAN (phosh_wwan_mm_new());
+        break;
+      case PHOSH_WWAN_BACKEND_OFONO:
+        priv->wwan = PHOSH_WWAN (phosh_wwan_ofono_new());
+        break;
+    }
+  }
+
+  g_return_val_if_fail (PHOSH_IS_WWAN (priv->wwan), NULL);
+  return priv->wwan;
+}
+
 /**
  * Returns the usable area in pixels usable by a client on the phone
  * display
@@ -1111,7 +1171,7 @@ phosh_shell_get_usable_area (PhoshShell *self, int *x, int *y, int *width, int *
   PhoshMonitor *monitor;
   PhoshMonitorMode *mode;
   int w, h;
-  int scale;
+  float scale;
 
   g_return_if_fail (PHOSH_IS_SHELL (self));
 
@@ -1120,11 +1180,11 @@ phosh_shell_get_usable_area (PhoshShell *self, int *x, int *y, int *width, int *
   mode = phosh_monitor_get_current_mode (monitor);
   g_return_if_fail (mode != NULL);
 
-  scale = monitor->scale ? monitor->scale : 1;
+  scale = MAX(1.0, phosh_monitor_get_fractional_scale (monitor));
 
-  g_debug ("Primary monitor %p scale is %d, transform is %d",
+  g_debug ("Primary monitor %p scale is %f, transform is %d",
            monitor,
-           monitor->scale,
+           scale,
            monitor->transform);
 
   switch (phosh_monitor_get_transform(monitor)) {
@@ -1133,18 +1193,18 @@ phosh_shell_get_usable_area (PhoshShell *self, int *x, int *y, int *width, int *
   case PHOSH_MONITOR_TRANSFORM_FLIPPED:
   case PHOSH_MONITOR_TRANSFORM_FLIPPED_180:
     w = mode->width / scale;
-    h = mode->height / scale - PHOSH_PANEL_HEIGHT - PHOSH_HOME_BUTTON_HEIGHT;
+    h = mode->height / scale - PHOSH_TOP_PANEL_HEIGHT - PHOSH_HOME_BUTTON_HEIGHT;
     break;
   default:
     w = mode->height / scale;
-    h = mode->width / scale - PHOSH_PANEL_HEIGHT - PHOSH_HOME_BUTTON_HEIGHT;
+    h = mode->width / scale - PHOSH_TOP_PANEL_HEIGHT - PHOSH_HOME_BUTTON_HEIGHT;
     break;
   }
 
   if (x)
     *x = 0;
   if (y)
-    *y = PHOSH_PANEL_HEIGHT;
+    *y = PHOSH_TOP_PANEL_HEIGHT;
   if (width)
     *width = w;
   if (height)
@@ -1169,7 +1229,6 @@ void
 phosh_shell_fade_out (PhoshShell *self, guint timeout)
 {
   PhoshShellPrivate *priv;
-  PhoshWayland *wl = phosh_wayland_get_default ();
   PhoshMonitorManager *monitor_manager;
 
   g_debug ("Fading out...");
@@ -1182,8 +1241,7 @@ phosh_shell_fade_out (PhoshShell *self, guint timeout)
     PhoshFader *fader;
     PhoshMonitor *monitor = phosh_monitor_manager_get_monitor (monitor_manager, i);
 
-    fader = phosh_fader_new (phosh_wayland_get_zwlr_layer_shell_v1 (wl),
-                             monitor->wl_output);
+    fader = phosh_fader_new (monitor);
     g_ptr_array_add (priv->faders, fader);
     gtk_widget_show (GTK_WIDGET (fader));
     if (timeout > 0)
@@ -1269,9 +1327,7 @@ phosh_shell_add_global_keyboard_action_entries (PhoshShell *self,
 
 void
 phosh_shell_remove_global_keyboard_action_entries (PhoshShell *self,
-                                                   const GActionEntry *entries,
-                                                   gint n_entries)
-
+                                                   GStrv       action_names)
 {
   PhoshShellPrivate *priv;
 
@@ -1279,9 +1335,9 @@ phosh_shell_remove_global_keyboard_action_entries (PhoshShell *self,
   priv = phosh_shell_get_instance_private (self);
   g_return_if_fail (priv->keyboard_events);
 
-  for (int i = 0; i < n_entries; i++) {
+  for (int i = 0; i < g_strv_length (action_names); i++) {
     g_action_map_remove_action (G_ACTION_MAP (priv->keyboard_events),
-                                entries[i].name);
+                                action_names[i]);
   }
 }
 
@@ -1378,5 +1434,62 @@ phosh_shell_set_state (PhoshShell          *self,
            enabled ? "Adding to" : "Removing from",
            str_state, str_new_flags);
 
-  g_object_notify_by_pspec (G_OBJECT (self), props[PHOSH_SHELL_PROP_SHELL_STATE]);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SHELL_STATE]);
+}
+
+void
+phosh_shell_lock (PhoshShell *self)
+{
+  g_return_if_fail (PHOSH_IS_SHELL (self));
+
+  phosh_shell_set_locked (self, TRUE);
+}
+
+
+void
+phosh_shell_unlock (PhoshShell *self)
+{
+  g_return_if_fail (PHOSH_IS_SHELL (self));
+
+  phosh_shell_set_locked (self, FALSE);
+}
+
+/**
+ * phosh_shell_get_locked:
+ * @self: The #PhoshShell singleton
+ *
+ * Returns: %TRUE if the shell is currently locked, otherwise %FALSE.
+ */
+gboolean
+phosh_shell_get_locked (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), FALSE);
+  priv = phosh_shell_get_instance_private (self);
+
+  return priv->locked;
+}
+
+/**
+ * phosh_shell_set_locked:
+ * @self: The #PhoshShell singleton
+ * @locked: %TRUE to lock the shell
+ *
+ * Lock the shell. We proxy to lockscreen-manager to avoid
+ * that other parts of the shell need to care about this
+ * abstraction.
+ */
+void
+phosh_shell_set_locked (PhoshShell *self, gboolean locked)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_if_fail (PHOSH_IS_SHELL (self));
+  priv = phosh_shell_get_instance_private (self);
+
+  if (locked == priv->locked)
+    return;
+
+  phosh_lockscreen_manager_set_locked (priv->lockscreen_manager, locked);
 }
