@@ -17,6 +17,7 @@
 #include "wifimanager.h"
 #include "shell.h"
 #include "phosh-wayland.h"
+#include "util.h"
 
 #include <NetworkManager.h>
 
@@ -55,6 +56,8 @@ struct _PhoshWifiManager
   char               *ssid;
 
   NMClient           *nmclient;
+  GCancellable       *cancel;
+
   /* The access point we're connected to */
   NMAccessPoint      *ap;
   /* The active connection (if it has a wifi device */
@@ -517,13 +520,13 @@ secret_agent_register_cb (GObject      *object,
   NMSecretAgentOld *agent = NM_SECRET_AGENT_OLD (object);
   g_autoptr(GError) error = NULL;
 
-  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
-
   if (!nm_secret_agent_old_register_finish (agent, result, &error)) {
-    g_warning ("Error registering network agent: %s", error->message);
+    g_message ("Error registering network agent: %s", error->message);
     return;
   }
 
+  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
+
   g_signal_connect_object (self->network_agent, "new-request",
                            G_CALLBACK (secret_request_new_cb),
                            self, G_CONNECT_SWAPPED);
@@ -557,17 +560,16 @@ on_nm_client_ready (GObject *obj, GAsyncResult *res, gpointer data)
 {
   g_autoptr(GError) err = NULL;
   PhoshWifiManager *self;
+  NMClient *client;
 
-  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (data));
-  self = PHOSH_WIFI_MANAGER (data);
-
-  self->nmclient = nm_client_new_finish (res, &err);
-  if (err) {
-    g_warning ("Failed to init NM: %s", err->message);
+  client = nm_client_new_finish (res, &err);
+  if (client == NULL) {
+    g_message ("Failed to init NM: %s", err->message);
     return;
   }
 
-  g_return_if_fail (NM_IS_CLIENT (self->nmclient));
+  self = PHOSH_WIFI_MANAGER (data);
+  self->nmclient = client;
 
   setup_network_agent (self);
   g_signal_connect_swapped (self->nmclient, "notify::wireless-enabled",
@@ -591,7 +593,8 @@ phosh_wifi_manager_constructed (GObject *object)
 {
   PhoshWifiManager *self = PHOSH_WIFI_MANAGER (object);
 
-  nm_client_new_async (NULL, on_nm_client_ready, self);
+  self->cancel = g_cancellable_new ();
+  nm_client_new_async (self->cancel, on_nm_client_ready, self);
 
   G_OBJECT_CLASS (phosh_wifi_manager_parent_class)->constructed (object);
 }
@@ -602,6 +605,14 @@ phosh_wifi_manager_dispose (GObject *object)
 {
   PhoshWifiManager *self = PHOSH_WIFI_MANAGER(object);
 
+  g_cancellable_cancel (self->cancel);
+  g_clear_object (&self->cancel);
+
+  if (self->ap) {
+    g_signal_handlers_disconnect_by_data (self->ap, self);
+    g_clear_object (&self->ap);
+  }
+
   g_clear_object (&self->network_agent);
   if (self->nmclient) {
     g_signal_handlers_disconnect_by_data (self->nmclient, self);
@@ -715,6 +726,25 @@ phosh_wifi_manager_get_enabled (PhoshWifiManager *self)
   return self->enabled;
 }
 
+void
+phosh_wifi_manager_set_enabled (PhoshWifiManager *self, gboolean enabled)
+{
+  int DEFAULT_TIMEOUT_MSEC = -1;
+
+  g_return_if_fail (PHOSH_IS_WIFI_MANAGER (self));
+
+  if (enabled == self->enabled)
+    return;
+
+  g_return_if_fail (NM_IS_CLIENT (self->nmclient));
+
+  nm_client_dbus_set_property (
+    self->nmclient, NM_DBUS_PATH, NM_DBUS_INTERFACE,
+    "WirelessEnabled", g_variant_new_boolean (enabled),
+    DEFAULT_TIMEOUT_MSEC, NULL, NULL, NULL
+  );
+}
+
 
 gboolean
 phosh_wifi_manager_get_present (PhoshWifiManager *self)
