@@ -94,7 +94,7 @@ main (int argc, char **argv)
                                          FALSE,
                                          NULL,
                                          now);
-  frame = phosh_notification_frame_new ();
+  frame = phosh_notification_frame_new (TRUE);
   phosh_notification_frame_bind_notification (PHOSH_NOTIFICATION_FRAME (frame),
                                               notification);
   g_signal_connect (frame, "empty", G_CALLBACK (empty), box);
@@ -114,7 +114,7 @@ main (int argc, char **argv)
                                          FALSE,
                                          NULL,
                                          now);
-  frame = phosh_notification_frame_new ();
+  frame = phosh_notification_frame_new (TRUE);
   phosh_notification_frame_bind_notification (PHOSH_NOTIFICATION_FRAME (frame),
                                               notification);
   g_signal_connect (frame, "empty", G_CALLBACK (empty), box);
